local function array_intersect(arr1, arr2)
    local values_set = {}
    for _, v in ipairs(arr1) do values_set[v] = true end
    local result, len = {}, 0
    for _, v in ipairs(arr2) do
        if values_set[v] then
            len = len + 1
            result[len] = v
        end
    end
    return result
end

ngx.req.clear_header("X-EW-Auth")
ngx.req.clear_header("X-EW-AuthEnvironment")

local searchengines = require './tulo-auth/useragents'

-- function to retrieve the config key for current domain
local function get_config_key()
    local config_key = string.gsub(ngx.var.http_host, 'www%.', '')
    return config_key
end

local config = require './tulo-auth/config'

local site_config = (get_config_key() ~= nil and config[get_config_key()] or nil)
local authorized = false

if site_config ~= nil then
    ngx.req.set_header("X-EW-AuthClient", site_config.sso.client)

    -- check if we should exclude current uri from auth-check
    for _, value in pairs(config.global.exclude) do
        local should_exclude = string.find(ngx.var.request_uri, value)
        if should_exclude then
            authorized = true
        end
    end

    -- check for whitelisted ip's
    local ip = ngx.var.remote_addr
    for _, wip in ipairs(config.global.ip_whitelist) do
        if wip == ip then
            authorized = true
            break
        end
    end

    -- check for whitelisted referers
    if ngx.var.http_referer ~= nil then
        local referer = ngx.var.http_referer
        for _, wr in ipairs(config.global.referer_whitelist) do
            if string.find(referer, wr) then
                authorized = true
            end
        end
    end

    -- search engines should be authorized
    local ua = ngx.req.get_headers()['User-Agent'];
    if ua ~= nil then
        for _, value in pairs(searchengines['patterns']) do
            local found = string.find(ua, value)
            if found ~= nil then
                authorized = true
                break
            end
        end
    end

    local cookie = ngx.var.cookie_ew_plus

    -- Check if we're using Tulo as auth system, then try fetch that cookie instead
    local tulo_auth_system = site_config.tulo;
    if tulo_auth_system then
        cookie = ngx.var.cookie_T_ID
    end

    -- check authorization of cookie
    if authorized == false and cookie ~= nil then
        local json = require 'cjson'
        local jwt = require 'luajwt'

        -- urldecode cookie information
        local unescaped_cookie = ngx.unescape_uri(cookie)

        -- fetch jwt from cookie
        local cookie_obj = json.decode(unescaped_cookie)

        if tulo_auth_system then
            -- Tulo auth check
            local jwt_data = jwt.decode(cookie_obj['signature'], site_config.sso.secret, true)

            if jwt_data ~= nil then

                if jwt_data.session.active then

                    local now = ngx.time()
                    local forward_url = site_config.sso.forward_url

                    if jwt_data.nbf and now >= jwt_data.nbf and now <= jwt_data.exp and jwt_data.iss == 'Tulo' and forward_url == jwt_data.aud then

                        local user_products = jwt_data.session.products

                        if site_config.products ~= nil and user_products ~= nil then

                            -- check that current user has access to the product
                            local active_products = table.getn(array_intersect(user_products, site_config.products))

                            if active_products > 0 then
                                authorized = true
                            else
                                authorized = 'noaccess'
                            end
                        end
                        -- authorized = true
                    end
                end
            else
                -- Do an extra check to see if we should reauth the session
                if ngx.var.cookie_ew_reauth == nil then

                    local expires = 900 -- 15 minutes
                    ngx.header["Set-Cookie"] = "ew_reauth=true; Path=/; Expires=" .. ngx.cookie_time(ngx.time() + expires)

                    return ngx.redirect(site_config.sso.forward_url .. '?code=siteCode&state=https://' .. ngx.var.host .. ngx.var.request_uri)
                end
            end
        end
    end

    ngx.req.set_header("X-EW-AuthEnvironment", site_config.sso.environment)
    ngx.req.set_header("X-EW-TuloAuth", tostring(tulo_auth_system))
end
